<?php
class Rating {
  // properties
  protected $svrating = 'mysql'; // 'mysql' / 'sqlite'
  static protected $conn = false;            // stores the connection to mysql
  public $affected_rows = 0;        // number of affected, or returned rows in SQL query
  protected $rater = '';                    // the user who rate, or its IP
  protected $nrrtg = 0;                 // if it is 1, the user can rate only one item in a day, 0 for multiple items
  public $rtgitems = 'rtgitems';        // Table to store items that are ranted
  public $rtgusers = 'rtgusers';             // Table that stores the users who rated
  protected $time;                // will store the current Timestamp
  protected $er_code =0; // to store and check erro code
  public $eror = false;          // to store and check for errors

  // constructor
  public function __construct() {
    // sets $svrating, $nrrtg, $rater, and $time properties
    if(defined('SVRATING')) $this->svrating = SVRATING;
    if(defined('NRRTG')) $this->nrrtg = NRRTG;
    if(defined('USRRATE') && USRRATE === 0) { if(defined('RATER')) $this->rater = RATER; }
    else if(isset($_COOKIE['rater']) && strlen($_COOKIE['rater']) > 1) $this->rater = $_COOKIE['rater'];
    else {
      $this->rater = $_SERVER['REMOTE_ADDR'];
      setcookie("rater", $this->rater, NEXTV);  // sets cookie with rater ip /name
    }
    $this->time = time();
  }

  // for connecting to mysql
  protected function setConn() {
    try {
      // Connect and create the PDO object
      if($this->svrating =='mysql') self::$conn = new PDO("mysql:host=".DBHOST."; dbname=".DBNAME, DBUSER, DBPASS);
      ELSE if($this->svrating =='sqlite') self::$conn = new PDO('sqlite:sqlite.db');

      // Sets to handle the errors in the ERRMODE_EXCEPTION mode
      self::$conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

      if($this->svrating =='mysql') self::$conn->exec('SET character_set_client="utf8",character_set_connection="utf8",character_set_results="utf8";'); // Sets encoding UTF-8
    }
    catch(PDOException $e) {
      $this->er_code = $e->getCode();
      $this->eror = 'Unable to connect: '. $e->getMessage();
    }
  }

  // Performs SQL queries
  public function sqlExec($sql, $vls) {
    if(self::$conn===false || self::$conn===NULL) $this->setConn();      // sets the connection to mysql
    $re = true;           // the value to be returned

    // if there is a connection set ($conn property not false)
    if(self::$conn !== false) {
      // gets the first word in $sql, to determine whenb SELECT query
      $ar_mode = explode(' ', trim($sql), 2);
      $mode = strtolower($ar_mode[0]);

      // performs the query and get returned data
      try {
        if($sqlprep = self::$conn->prepare($sql)) {
          // execute query
          if($sqlprep->execute($vls)) {
            // if $mode is 'select', gets the result_set to return
            if($mode == 'select') {
              $re =[];
              // if fetch() returns at least one row (not false), adds the rows in $re for return
              if(($row = $sqlprep->fetch(PDO::FETCH_ASSOC)) !== false){
                do {
                  // check each column if it has numeric value, to convert it from "string"
                  foreach($row AS $k=>$v) {
                    if(is_numeric($v)) $row[$k] = $v + 0;
                  }
                  $re[] = $row;
                }
                while($row = $sqlprep->fetch(PDO::FETCH_ASSOC));
              }
              $this->affected_rows = count($re);                   // number of returned rows
            }
          }
          else $this->eror = 'Cannot execute the sql query';
        }
        else {
          $eror = self::$conn->errorInfo();
          $this->eror = 'Error: '. $eror[2];
        }
      }
      catch(PDOException $e) {
        $this->er_code = $e->getCode();
        if($this->er_code !=23000) $this->eror = $e->getMessage();
      }
    }

    // sets to return false in case of error
    if($this->eror !== false) { echo $this->eror; $re = false; }
    return $re;
  }

  // returns JSON string with item:['totalrate', 'nrrates', renot] for each element in $items array 
  public function getRating($items, $rate = '') {
    $rtgstdy = $this->rtgstdyDb();

    // if $rate not empty, perform to register the rating, $items contains one item to rate
    if(!empty($rate)) {
      // if $rater empty means user not loged
      if($this->rater === '') return "alert('Vote Not registered.\\nYou must be logged in to can rate')";

      // if already rated, returns JSON from which JS alert message and will reload the page
      // else, accesses the method to add the new rating
      if(in_array($items[0], $rtgstdy)) return '{"'.$items[0].'":[0,0,3]}';
      else if($this->svrating =='mysql') $this->setRtgDb($items, $rate, $rtgstdy); // add the new rate in mysql
      else if($this->svrating =='sqlite') $this->setRtgSQLite($items, $rate, $rtgstdy); // add the new rate in SQLite

     array_push($rtgstdy, $items[0]);        // adds curent item as rated
   }

    // if $nrrtg is 1, and $rtgstdy has item, set $setrated=1 (user already ranked today)
    // else, user can rate multiple items, after Select is checked if already rated the existend $item
    $setrated = ($this->nrrtg === 1 && count($rtgstdy) > 0) ? 1 : 0;

    // get array with items and their ratings from mysql or TXT file
    $rtgitems = $this->getRtgDb($items, $rtgstdy, $setrated);

    return json_encode($rtgitems);
  }

  // insert /update rating item in #rtgitems, delete rows in $rtgusers which are less than $time, insert $rater in $rtgusers
  protected function setRtgDb($items, $rate, $rtgstdy) {
    $this->sqlExec("INSERT INTO `$this->rtgitems` (`item`, `totalrate`) VALUES ('".$items[0]."', $rate) ON DUPLICATE KEY UPDATE `totalrate`=`totalrate`+$rate, `nrrates`=`nrrates`+1", []);

    $vls =['nextv'=>$this->time];
    $this->sqlExec("DELETE FROM `$this->rtgusers` WHERE `nextv`<:nextv", $vls);

    $vls =['nextv'=>NEXTV, 'rater'=>$this->rater, 'item'=>$items[0]];
    $this->sqlExec("INSERT INTO `$this->rtgusers` (`nextv`, `rater`, `item`) VALUES (:nextv, :rater, :item)", $vls);
  }

  // insert /update rating item in #rtgitems for SQLite, delete rows in $rtgusers which are not from today, insert $rater in $rtgusers
  protected function setRtgSQLite($items, $rate, $rtgstdy) {
    $vls =['item'=>$items[0], 'rate'=>$rate];
    $this->sqlExec("INSERT INTO `$this->rtgitems` (`item`, `totalrate`) VALUES (:item, :rate)", $vls);

    //if item already added, performs update
    if($this->er_code ==23000) $this->sqlExec("UPDATE `$this->rtgitems` SET `totalrate`=`totalrate`+$rate, `nrrates`=`nrrates`+1 WHERE item='".$items[0]."'", []);

    $vls =['nextv'=>$this->time];
    $this->sqlExec("DELETE FROM `$this->rtgusers` WHERE `nextv`<:nextv", $vls);

    $vls =['nextv'=>NEXTV, 'rater'=>$this->rater, 'item'=>$items[0]];
    $this->sqlExec("INSERT INTO `$this->rtgusers` (`nextv`, `rater`, `item`) VALUES (:nextv, :rater, :item)", $vls);
  }

  // select 'totalrate' and 'nrrates' of each element in $items, $rtgstdy stores items rated by the user today
  // returns array with item:['totalrate', 'nrrates', renot] for each element in $items array
  protected function getRtgDb($items, $rtgstdy, $setrated) {
    $re = array_fill_keys($items, [0,0,$setrated]);    // makes each value of $items as key with an[0,0,0]

    function addSlhs($elm){return "'".$elm."'";}      // function to be used in array_map(), adds "'" to each $elm
    $resql = $this->sqlExec("SELECT * FROM `$this->rtgitems` WHERE `item` IN(".implode(',', array_map('addSlhs', $items)).")", []);
    if($this->affected_rows > 0) {
      for($i=0; $i<$this->affected_rows; $i++) {
        $rated = in_array($resql[$i]['item'], $rtgstdy) ? $setrated + 1 : $setrated;   // add 1 if the item was ranked by the user today
        $re[$resql[$i]['item']] = [$resql[$i]['totalrate'], $resql[$i]['nrrates'], $rated];
      }
    }

    return $re;
  }

  // returns from mysql an array with items ranked by the user
  protected function rtgstdyDb() {
    $rtgstdy = [];
    $vls =['nextv'=>$this->time, 'rater'=>$this->rater];
    $resql = $this->sqlExec("SELECT `item` FROM `$this->rtgusers` WHERE `nextv`>:nextv AND `rater`=:rater", $vls);
    if($this->affected_rows > 0) {
      for($i=0; $i<$this->affected_rows; $i++) {
        $rtgstdy[] = $resql[$i]['item'];
      }
    }

    return $rtgstdy;
  }
}